#include "../Include/AppSystem.h"

//-----------------------------------------------------------------------------
// Name : DXSPRITE_INFO()
//-----------------------------------------------------------------------------
DXSPRITE_INFO::DXSPRITE_INFO()
{
	angle			= 0;
	RotationCenter	= D3DXVECTOR2( 0, 0 );
	scale			= D3DXVECTOR2( 1.0f, 1.0f );
	ScalingCenter	= D3DXVECTOR2( 0, 0 );
	color			= COLOR_XRGB( 255, 255, 255 );
	blend			= BLEND_NORMAL;
	SetRect( &rect, 0, 0, 0, 0 );
}

//-----------------------------------------------------------------------------
// Name : ~DXSPRITE_INFO()
//-----------------------------------------------------------------------------
DXSPRITE_INFO::~DXSPRITE_INFO()
{
	d3d.texture.Delete( TexObj );
}

//-----------------------------------------------------------------------------
// Name : CDXSpriteManager()
//-----------------------------------------------------------------------------
CDXSpriteManager::CDXSpriteManager()
{
	m_pSprite = 0;
}

//-----------------------------------------------------------------------------
// Name : ~CDXSpriteManager()
//-----------------------------------------------------------------------------
CDXSpriteManager::~CDXSpriteManager()
{
	RELEASE( m_pSprite );
}

//-----------------------------------------------------------------------------
// Name : Init()
// Desc : 
//-----------------------------------------------------------------------------
bool CDXSpriteManager::Init()
{
	static HRESULT hr;
	hr = D3DXCreateSprite( d3d.m_pD3DDev, &m_pSprite );
	if ( FAILED( hr ) ) {
		ErrLog( "CDXSpriteManager::Init() : D3DXCreateSprite()\n"
				"		%s", DXGetErrorString9( hr ) );
		MesBox( "G[", "DXXvCg̏Ɏs܂" );
		return false;
	}

	return true;
}

//-----------------------------------------------------------------------------
// Name : OnLostDevice
// Desc : foCX̏
//-----------------------------------------------------------------------------
bool CDXSpriteManager::OnLostDevice()
{
	if ( m_pSprite ) {
		static HRESULT hr;
		hr = m_pSprite->OnLostDevice();
		if ( FAILED( hr ) ) {
			ErrLog( "CDXSpriteManager::OnLostDevice() : OnLostDevice()\n"
					"		%s", DXGetErrorString9( hr ) );
			MesBox( "G[", "DXXvCgEfoCX̏Ɏs܂" );
			return false;
		}
	}

	return true;
}

//-----------------------------------------------------------------------------
// Name : OnResetDevice
// Desc : foCX̕
//-----------------------------------------------------------------------------
bool CDXSpriteManager::OnResetDevice()
{
	if ( m_pSprite ) {
		static HRESULT hr;
		hr = m_pSprite->OnResetDevice();
		if ( FAILED( hr ) ) {
			ErrLog( "CDXSpriteManager::OnResetDevice() : OnResetDevice()\n"
					"		%s", DXGetErrorString9( hr ) );
			MesBox( "G[", "DXXvCgEfoCX̕Ɏs܂" );		
			return false;
		}
	}

	return true;
}

//-----------------------------------------------------------------------------
// Name : GetpSprite()
// Desc : XvCg̎擾
//-----------------------------------------------------------------------------
bool CDXSpriteManager::GetpSprite( LPD3DXSPRITE *ppSprite )
{
	if ( !m_pSprite ) return false;

	*ppSprite = m_pSprite;
	return true;
}


//-----------------------------------------------------------------------------
// Name : Begin()
// Desc : `Jn
//-----------------------------------------------------------------------------
bool CDXSpriteManager::Begin()
{
	if ( FAILED( m_pSprite->Begin( 
		D3DXSPRITE_SORT_DEPTH_FRONTTOBACK | D3DXSPRITE_ALPHABLEND ) ) )
		return false;

	return true;
}

//-----------------------------------------------------------------------------
// Name : End()
// Desc : `I
//-----------------------------------------------------------------------------
bool CDXSpriteManager::End()
{
	if ( FAILED( m_pSprite->End() ) )
		return false;

	return true;
}

//-----------------------------------------------------------------------------
// Name : Create()
// Desc : XvCg̍쐬
// Argu : obj = XvCgEIuWFNg
//		  filename = t@C
//		  InvalidColor = F
//-----------------------------------------------------------------------------
bool CDXSpriteManager::Create( DXSPROBJ &obj, char* filename, D3DCOLOR ColorKey )
{
	// IuWFNg̍쐬
	m_vInfo.Create( obj );
	DXSPRITE_INFO *pInfo = m_vInfo.GetpObject( obj );

	// eNX`̍쐬
	if ( !d3d.texture.Create( pInfo->TexObj, filename, ColorKey ) ) {
		m_vInfo.Delete( obj );
		ErrLog( "CDXSpriteManager::Create() : Create()\n"
				"		%s" );
		return false;
	}

	// XvCg̏TCYC[WTCYƓɂ
	SIZE ImageSize;
	d3d.texture.GetImageSize( ImageSize, pInfo->TexObj );
	::SetRect( &( pInfo->rect ), 0, 0, ImageSize.cx, ImageSize.cy );

	return true;
}

//-----------------------------------------------------------------------------
// Name : Create()
// Desc : XvCg쐬@
// Argu : obj = XvCgEIuWFNg
//		  filename = t@C
//		  InvalidColor = F
//-----------------------------------------------------------------------------
bool CDXSpriteManager::Create( DXSPROBJ &obj, TEXOBJ TexObj )
{
	// TEXOBJ̃`FbN
	if ( !d3d.texture.Check( TexObj ) ) return false;
	
	m_vInfo.Create( obj );
	DXSPRITE_INFO *pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	// eNX`̃Rs[
	if ( !d3d.texture.Duplicate( pInfo->TexObj, TexObj ) ) {
		ErrLog( "CDXSpriteManager::Create() : Duplicate()" );
		return false;
	}

	// XvCg̏TCY摜TCYƓɂ
	SIZE ImageSize;
	d3d.texture.GetImageSize( ImageSize, pInfo->TexObj );
	::SetRect( &( pInfo->rect ), 0, 0, ImageSize.cx, ImageSize.cy );

	return true;
}

//-----------------------------------------------------------------------------
// Name : Draw()
// Desc : XvCg̕\
//-----------------------------------------------------------------------------
bool CDXSpriteManager::Draw( DXSPROBJ obj, float x, float y, float z )
{	
	static HRESULT hr;

	DXSPRITE_INFO *pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	// uhݒ
	switch ( pInfo->blend ) {
	case BLEND_NORMAL:
		d3d.m_pD3DDev->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
		d3d.m_pD3DDev->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );	
		break;
	case BLEND_ADD:
		d3d.m_pD3DDev->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_ONE );
		d3d.m_pD3DDev->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_ONE );
		break;
	}
	
	// gXtH[s
	D3DXVECTOR2 Translation( x, y );
	D3DXMATRIX mat;
	D3DXMatrixTransformation2D( &mat, &( pInfo->ScalingCenter ), 0, 
								&( pInfo->scale ), &( pInfo->RotationCenter ), 
								pInfo->angle, &Translation );

	// gXtH[Zbg
	m_pSprite->SetTransform( &mat );

	// `ʒu
	D3DXVECTOR3 Position( 0, 0, z );

	// `

	hr = m_pSprite->Draw( d3d.texture.GetpTexture( pInfo->TexObj ), &( pInfo->rect ), 
						  &D3DXVECTOR3( pInfo->ScalingCenter.x, pInfo->ScalingCenter.y, 0 ), 
						  &Position, pInfo->color );
	if ( FAILED( hr ) ) {
		ErrLog( "CDXSpriteManager::Draw() : Draw()\n"
				"		%s", DXGetErrorString9( hr ) );
		return false;
	}

	m_pSprite->Flush();

	return true;
}

//-----------------------------------------------------------------------------
// Name : Delete()
// Desc : XvCg̍폜
// Argu : obj = XvCgEIuWFNg
//-----------------------------------------------------------------------------
bool CDXSpriteManager::Delete( DXSPROBJ &obj )
{
	if ( m_vInfo.Delete( obj ) ) return true;
	return false;
}

//-----------------------------------------------------------------------------
// Name : Duplicate
// Desc : XvCg̕
// Argu : dest = 
//		  source = 
//-----------------------------------------------------------------------------
bool CDXSpriteManager::Duplicate( DXSPROBJ &dest, DXSPROBJ source )
{
	// ƕ悪ꍇAȂ
	if ( dest == source ) return true;

	DXSPRITE_INFO *pSourceInfo	= m_vInfo.GetpObject( source );
	if ( pSourceInfo == 0 ) return false;
	
	m_vInfo.Create( dest );
	DXSPRITE_INFO *pDestInfo	= m_vInfo.GetpObject( dest );

	TEXOBJ TexObj;
	d3d.texture.Duplicate( TexObj, pSourceInfo->TexObj );

	// XvCgRs[
	*pDestInfo = *pSourceInfo;
	pDestInfo->TexObj = TexObj;

	return true;
}

//-----------------------------------------------------------------------------
// Name : SetAngle
// Desc : XvCg̊px̐ݒ
// Argu : obj = XvCgEIuWFNg
//		  angle = px
//		  cx	= ]̒S( XW )
//		  cy	= ]̒S( YW )
//-----------------------------------------------------------------------------
bool CDXSpriteManager::SetRotate( DXSPROBJ obj, float angle, float cx, float cy )
{
	DXSPRITE_INFO *pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	pInfo->angle			= D3DXToRadian( angle );
	pInfo->RotationCenter.x = cx;
	pInfo->RotationCenter.y = cy;

	return true;
}

//-----------------------------------------------------------------------------
// Name : SetScale
// Desc : XvCg̃XP[̐ݒ
// Argu : obj = XvCgEIuWFNg
//		  ScaleX = {
//		  ScaleY = c{
//-----------------------------------------------------------------------------
bool CDXSpriteManager::SetScale( DXSPROBJ obj, float sx, float sy, float cx, float cy  )
{
	DXSPRITE_INFO *pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	pInfo->ScalingCenter.x	= cx;
	pInfo->ScalingCenter.y	= cy;
	pInfo->scale.x			= sx;
	pInfo->scale.y			= sy;

	return true;
}
//-----------------------------------------------------------------------------
// Name : SetBlendColor
// Desc : XvCg̃uhF̐ݒ
// Argu : obj = XvCgEIuWFNg
//		  ScaleX = {
//		  ScaleY = c{
//-----------------------------------------------------------------------------
bool CDXSpriteManager::SetBlendColor( DXSPROBJ obj, BLEND_MODE mode, D3DCOLOR color )
{
	DXSPRITE_INFO *pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	pInfo->blend = mode;
	pInfo->color = color;

	return true;
}

//-----------------------------------------------------------------------------
// Name : SetTextureRect
// Desc : XvCg̕\
//-----------------------------------------------------------------------------
bool CDXSpriteManager::SetTextureRect( DXSPROBJ obj, int left, int top, int right, 
									  int bottom )
{
	DXSPRITE_INFO *pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;
	
	pInfo->rect.left	= left;
	pInfo->rect.top		= top;
	pInfo->rect.right	= right;
	pInfo->rect.bottom	= bottom;

	return true;
}
