#include "../Include/AppSystem.h"

//-----------------------------------------------------------------------------
// Name : LINE_INFO()
//-----------------------------------------------------------------------------
LINE_INFO::LINE_INFO()
{
	Color		= COLOR_XRGB( 0, 0, 0 );
	fWidth		= 1.0f;
	bAntialias	= false;
	dwPattern	= 1;
	bGLLines	= false;
	fScale		= 1.0f;
}

//-----------------------------------------------------------------------------
// Name : ~LINE_INFO()
//-----------------------------------------------------------------------------
LINE_INFO::~LINE_INFO()
{
	d3d.texture.Delete( TexObj );
}

//-----------------------------------------------------------------------------
// Name : CLine()
//-----------------------------------------------------------------------------
CLine::CLine()
{
	m_pLine = 0;
}

//-----------------------------------------------------------------------------
// Name : ~CLine()
//-----------------------------------------------------------------------------
CLine::~CLine()
{
	RELEASE( m_pLine );
}

//-----------------------------------------------------------------------------
// Name : OnLostDevice()
//-----------------------------------------------------------------------------
void CLine::OnLostDevice()
{
	if ( m_pLine )
		m_pLine->OnLostDevice();
}

//-----------------------------------------------------------------------------
// Name : OnResetDevice()
//-----------------------------------------------------------------------------
void CLine::OnResetDevice()
{
	if ( m_pLine )
		m_pLine->OnResetDevice();
}

//-----------------------------------------------------------------------------
// Name : Init()
//-----------------------------------------------------------------------------
bool CLine::Init()
{
	if ( FAILED( D3DXCreateLine( d3d.m_pD3DDev, &m_pLine ) ) )
		return false;

	return true;
}

//-----------------------------------------------------------------------------
// Name : Create()
//-----------------------------------------------------------------------------
bool CLine::Create( BASE_LINEOBJ &obj )
{
	m_vInfo.Create( obj );

	return true;
}

//-----------------------------------------------------------------------------
// Name : SetTexture()
//-----------------------------------------------------------------------------
bool CLine::SetTexture( BASE_LINEOBJ obj, const char *filename )
{
	static LINE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	// eNX`̍쐬
	if ( !d3d.texture.Create( pInfo->TexObj, filename ) ) {
		m_vInfo.Delete( obj );
		return false;
	}

	return true;
}

//-----------------------------------------------------------------------------
// Name : SetVertex3D()
//-----------------------------------------------------------------------------
bool CLine::SetVertex3D( LINE3DOBJ obj, D3DXVECTOR3 vec )
{
	static LINE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;
	
	pInfo->v3DVertex.push_back( vec );

	return true;
}

//-----------------------------------------------------------------------------
// Name : SetVertex2D()
//-----------------------------------------------------------------------------
bool CLine::SetVertex2D( LINE2DOBJ obj, D3DXVECTOR2 vec )
{
	static LINE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	pInfo->v2DVertex.push_back( vec );

	return true;
}

//-----------------------------------------------------------------------------
// Name : Delete3DVertex()
//-----------------------------------------------------------------------------
bool CLine::Delete3DVertex( LINE3DOBJ obj, int n )
{
	static LINE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	for ( int i = 0; i < n; i++ ) {
		if ( pInfo->v3DVertex.size() <= 0 ) break;
		pInfo->v3DVertex.pop_back();
	}
	return true;
}

//-----------------------------------------------------------------------------
// Name : Delete2DVertex()
//-----------------------------------------------------------------------------
bool CLine::Delete2DVertex( LINE2DOBJ obj, int n )
{
	static LINE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	for ( int i = 0; i < n; i++ ) {
		if ( pInfo->v2DVertex.size() <= 0 ) break;
		pInfo->v2DVertex.pop_back();
	}
	return true;
}

//-----------------------------------------------------------------------------
// Name : SetColor()
//-----------------------------------------------------------------------------
bool CLine::SetColor( BASE_LINEOBJ obj, D3DCOLOR color )
{
	static LINE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	pInfo->Color = color;

	return true;
}

//-----------------------------------------------------------------------------
// Name : Draw2D()
//-----------------------------------------------------------------------------
bool CLine::Draw2D( LINE2DOBJ obj )
{
	static LINE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	if ( pInfo->v2DVertex.size() <= 1 ) return false;

	static HRESULT hr;

	// p[^̐ݒ
	m_pLine->SetPattern( pInfo->dwPattern );
	m_pLine->SetWidth( pInfo->fWidth );
	m_pLine->SetAntialias( pInfo->bAntialias );
	m_pLine->SetGLLines( pInfo->bGLLines );
	m_pLine->SetPatternScale( pInfo->fScale );

	if ( FAILED( m_pLine->Begin() ) ) return false;
	
	d3d.m_pD3DDev->SetTexture( 0, d3d.texture.GetpTexture( pInfo->TexObj ) );

	//eNX`Xe[WXe[g
	d3d.m_pD3DDev->SetTextureStageState( 0, D3DTSS_COLOROP, D3DTOP_MODULATE );
	d3d.m_pD3DDev->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
	d3d.m_pD3DDev->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTSS_COLOROP );
	d3d.m_pD3DDev->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_DISABLE );

	//`
	hr = m_pLine->Draw( &pInfo->v2DVertex[0], pInfo->v2DVertex.size(), pInfo->Color);
	if ( FAILED( hr ) ) {
		return false;
	}

	d3d.m_pD3DDev->SetTexture( 0, 0 );

	m_pLine->End();

	return true;
}

//-----------------------------------------------------------------------------
// Name : Draw3D()
//-----------------------------------------------------------------------------
bool CLine::Draw3D( LINE3DOBJ obj )
{
	if ( !m_vInfo.Check( obj.n ) ) return false;

	LINE_INFO *pInfo = m_vInfo.GetpObject( obj.n );
	if ( pInfo->v3DVertex.size() <= 1 ) return false;

	HRESULT hr;

	// p[^̐ݒ
	m_pLine->SetPattern( pInfo->dwPattern );
	m_pLine->SetWidth( pInfo->fWidth );
	m_pLine->SetAntialias( pInfo->bAntialias );
	m_pLine->SetGLLines( pInfo->bGLLines );
	m_pLine->SetPatternScale( pInfo->fScale );

	if ( FAILED( m_pLine->Begin() ) ) return false;	

	d3d.m_pD3DDev->SetTexture( 0, d3d.texture.GetpTexture( pInfo->TexObj ) );

	//eNX`Xe[WXe[g
	d3d.m_pD3DDev->SetTextureStageState( 0, D3DTSS_COLOROP, D3DTOP_MODULATE );
	d3d.m_pD3DDev->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
	d3d.m_pD3DDev->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTSS_COLOROP );
	d3d.m_pD3DDev->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_DISABLE );

	// švZ
	D3DXMATRIXA16 mat, matView, matProj;
	
	D3DXMatrixIdentity(&mat);
	d3d.m_pD3DDev->SetTransform( D3DTS_WORLD, &mat );
	d3d.m_pD3DDev->GetTransform( D3DTS_VIEW, &matView );
	d3d.m_pD3DDev->GetTransform( D3DTS_PROJECTION, &matProj );
	mat = mat * matView * matProj;

	// `
	hr = m_pLine->DrawTransform( &pInfo->v3DVertex[0], pInfo->v3DVertex.size(), 
			&mat, pInfo->Color );
	if ( FAILED( hr ) ) return false;

	d3d.m_pD3DDev->SetTexture( 0, 0 );

	m_pLine->End();

	return true;
}

//-----------------------------------------------------------------------------
// Name : SetPattern()
//-----------------------------------------------------------------------------
bool CLine::SetPattern( BASE_LINEOBJ obj, DWORD dwPattern )
{
	if ( !m_vInfo.Check( obj.n ) ) return false;

	m_vInfo.GetpObject( obj.n )->dwPattern = dwPattern;
	return true;
}

//-----------------------------------------------------------------------------
// Name : SetWidth()
//-----------------------------------------------------------------------------
bool CLine::SetWidth( BASE_LINEOBJ obj, float fWidth )
{
	if ( !m_vInfo.Check( obj.n ) ) return false;

	m_vInfo.GetpObject( obj.n )->fWidth = fWidth;	//̑Zbg
	return true;
}

//-----------------------------------------------------------------------------
// Name : SetAntialias()
//-----------------------------------------------------------------------------
bool CLine::SetAntialias( BASE_LINEOBJ obj, bool bl )
{
	if ( !m_vInfo.Check( obj.n ) ) return false;

	m_vInfo.GetpObject( obj.n )->bAntialias = bl;	//A`GCAVO
	return true;
}

//-----------------------------------------------------------------------------
// Name : SetGLLines()
//-----------------------------------------------------------------------------
bool CLine::SetGLLines( BASE_LINEOBJ obj, bool bl )
{
	if ( !m_vInfo.Check( obj.n ) ) return false;

	m_vInfo.GetpObject( obj.n )->bGLLines = bl;
	return true;
}

//-----------------------------------------------------------------------------
// Name : SetPatternScale()
//-----------------------------------------------------------------------------
bool CLine::SetPatternScale( BASE_LINEOBJ obj, float fScale )
{
	if ( !m_vInfo.Check( obj.n ) ) return false;

	m_vInfo.GetpObject( obj.n )->fScale = fScale;
	return true;
}

//-----------------------------------------------------------------------------
// Name : GetpLineInfo()
//-----------------------------------------------------------------------------
const LINE_INFO* CLine::GetpLineInfo( BASE_LINEOBJ obj )
{
	if ( !m_vInfo.Check( obj.n ) ) return 0;

	return m_vInfo.GetpObject( obj.n );
}

//-----------------------------------------------------------------------------
// Name : Delete()
//-----------------------------------------------------------------------------
bool CLine::Delete( BASE_LINEOBJ &obj )
{
	if ( m_vInfo.Delete( obj.n ) ) {
		obj.n = -1;
		return true;
	}
	return false;
}

//-----------------------------------------------------------------------------
// Name : ClearVertex()
//-----------------------------------------------------------------------------
bool CLine::ClearVertex( BASE_LINEOBJ &obj )
{
	if ( m_vInfo.Check( obj.n ) ) {
		LINE_INFO *pInfo = m_vInfo.GetpObject( obj.n );
		pInfo->v2DVertex.clear();
		pInfo->v3DVertex.clear();
	}
	return false;
}