#include "../Include/AppSystem.h"

//-----------------------------------------------------------------------------
// Name : SPRITE_INFO()
//-----------------------------------------------------------------------------
SPRITE_INFO::SPRITE_INFO()
{
	pos				= D3DXVECTOR2( 0, 0 );
	UpLeftColor		= COLOR_XRGB( 255,255,255 );
	UpRightColor	= COLOR_XRGB( 255,255,255 );
	LowRightColor	= COLOR_XRGB( 255,255,255 );
	LowLeftColor	= COLOR_XRGB( 255,255,255 );
	angle			= 0;						// ]px
	RotateCenter	= D3DXVECTOR2( 0.0f, 0.0f );	// ]̒S
	scale			= D3DXVECTOR2( 1.0f, 1.0f );	// XP[
	ScaleCenter		= D3DXVECTOR2( 0.0f, 0.0f );	// XP[̒S
	blend			= BLEND_NORMAL;				// uh[h
}

//-----------------------------------------------------------------------------
// Name : ~SPRITE_INFO()
//-----------------------------------------------------------------------------
SPRITE_INFO::~SPRITE_INFO()
{
	d3d.texture.Delete( TexObj );
}

//-----------------------------------------------------------------------------
// Name : Create()
// Desc : XvCg̍쐬
// Argu : obj			= SPROBJ IuWFNg
//		  filename		= t@C
//		  InvalidColor	= F
//-----------------------------------------------------------------------------
bool CSpriteManager::Create( SPROBJ &obj, char *filename, D3DCOLOR Colorless )
{
	m_vInfo.Create( obj );
	static SPRITE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	// eNX`̍쐬
	if ( !d3d.texture.Create( pInfo->TexObj, filename, Colorless ) ) {
		m_vInfo.Delete( obj );
		ErrLog( "CSpriteManager::Create() : Create()\n"
			    "		%s", filename );
		return false;
	}
	
	// XvCg̏TCYeNX`ETCYɂ
	SIZE ImageSize;
	d3d.texture.GetImageSize( ImageSize, pInfo->TexObj );
	::SetRect( &pInfo->TextureRect, 0, 0, ImageSize.cx, ImageSize.cy );

	return true;
}

//-----------------------------------------------------------------------------
// Name : Create()
// Desc : XvCg̍쐬
// Argu : obj			= SPROBJ IuWFNg
//		  TexObj		= TEXOBJ IuWFNg
//-----------------------------------------------------------------------------
bool CSpriteManager::Create( SPROBJ &obj, TEXOBJ TexObj )
{
	m_vInfo.Create( obj );

	static SPRITE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	if ( !d3d.texture.Duplicate( pInfo->TexObj, TexObj ) ) {
		ErrLog( "CSpriteManager::Create() : Duplicate()" );
		m_vInfo.Delete( obj );
		return false;
	}

	// XvCg̏TCYeNX`ETCYɂ
	SIZE ImageSize;
	d3d.texture.GetImageSize( ImageSize, pInfo->TexObj );
	::SetRect( &pInfo->TextureRect, 0, 0, ImageSize.cx, ImageSize.cy );

	return true;
}

//-----------------------------------------------------------------------------
// Name : Duplicate()
// Desc : XvCg̕
// Argu : dest		= SPROBJ IuWFNg
//		  source	= SPROBJ IuWFNg
//-----------------------------------------------------------------------------
bool CSpriteManager::Duplicate( SPROBJ &dest, SPROBJ source )
{
	// ƕ悪ꍇAȂ
	if ( dest == source ) return true;

	static SPRITE_INFO *pSourceInfo;
	pSourceInfo = m_vInfo.GetpObject( source );
	if ( pSourceInfo == 0 ) return false;

	m_vInfo.Create( dest );

	static SPRITE_INFO *pDestInfo;
	pDestInfo = m_vInfo.GetpObject( dest );
	if ( pDestInfo == 0 ) return false;

	TEXOBJ TexObj;
	d3d.texture.Duplicate( TexObj, pSourceInfo->TexObj );

	// XvCgRs[
	*pDestInfo = *pSourceInfo;
	pDestInfo->TexObj = TexObj;

	return true;
}

//-----------------------------------------------------------------------------
// Name : Draw()
// Desc : XvCg̍쐬
// Argu : obj	= SPROBJ IuWFNg
//		  x, y	= `悷W
//-----------------------------------------------------------------------------
bool CSpriteManager::Draw( SPROBJ obj, float x, float y, float z )
{
	static SPRITE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	// eNX`TCY
	// D3DSURFACE_DESC desc;
	// d3d.texture.GetpTexture( pInfo->TexObj )->GetLevelDesc( 0, &desc );
	// desc.Width   desc.Height

	// 摜TCY̎擾
	SIZE ImageSize, TextureSize;
	d3d.texture.GetImageSize( ImageSize, pInfo->TexObj );
	d3d.texture.GetTextureSize( TextureSize, pInfo->TexObj );
/*
	D3DXVECTOR2 vScaling;
	vScaling.x = (float) ImageSize.cx / TextureSize.cx;
	vScaling.y = (float) ImageSize.cy / TextureSize.cy;
*/
	// eNX`UVWH̐ݒ
	float u = (float) pInfo->TextureRect.left	/ TextureSize.cx;
	float v = (float) pInfo->TextureRect.top	/ TextureSize.cy;
	float w = (float) pInfo->TextureRect.right	/ TextureSize.cx;
	float h = (float) pInfo->TextureRect.bottom / TextureSize.cy;
/*
	D3DXMATRIX mat;	
	D3DXVECTOR2 Translation( x, y );
	D3DXMatrixTransformation2D( &mat, &( pInfo->ScaleCenter ), 0, &( pInfo->scale ), 
		&( pInfo->RotateCenter ), -D3DXToRadian( pInfo->angle ), &Translation );
*/
	D3DXMATRIX mat;	
	D3DXVECTOR2 Translation( x, y );
	D3DXMatrixTransformation2D( &mat, &( pInfo->ScaleCenter ), 0, &( pInfo->scale ), 
		&( pInfo->RotateCenter ), -D3DXToRadian( pInfo->angle ), &Translation );

	// eNX`ETCY̎擾
	SIZE size;
	size.cx = pInfo->TextureRect.right;
	size.cy = pInfo->TextureRect.bottom;

	// _W̌vZ
	// SWgk邽߂ɁA炩ߒSW̐l[JWĂ
	D3DXVECTOR2 vtex[4];
	vtex[0] = D3DXVECTOR2( 0 - pInfo->ScaleCenter.x,		0 - pInfo->ScaleCenter.y );
	vtex[1] = D3DXVECTOR2( size.cx - pInfo->ScaleCenter.x,	0 - pInfo->ScaleCenter.y );
	vtex[2] = D3DXVECTOR2( size.cx - pInfo->ScaleCenter.x,	size.cy - pInfo->ScaleCenter.y );
	vtex[3] = D3DXVECTOR2( 0 - pInfo->ScaleCenter.x,		size.cy - pInfo->ScaleCenter.y );
	
	for ( int i = 0; i < 4; i++ )
		D3DXVec2TransformCoord( &vtex[i], &vtex[i], &mat );

	D3DTLVERTEX vertex[] = {
		{ -0.5+vtex[0].x, -0.5+vtex[0].y, z, 1.0f, pInfo->UpLeftColor,   0.0f, u,		v },
		{ -0.5+vtex[1].x, -0.5+vtex[1].y, z, 1.0f, pInfo->UpRightColor,  0.0f, u + w,	v },
		{ -0.5+vtex[2].x, -0.5+vtex[2].y, z, 1.0f, pInfo->LowRightColor, 0.0f, u + w,	v + h },
		{ -0.5+vtex[3].x, -0.5+vtex[3].y, z, 1.0f, pInfo->LowLeftColor,  0.0f, u,		v + h }
	};

/*
	D3DTLVERTEX vertex[] = {
		{ vtex[0].x, vtex[0].y, z, 1.0f, pInfo->UpLeftColor,   0.0f, u,	v },
		{ vtex[1].x, vtex[1].y, z, 1.0f, pInfo->UpRightColor,  0.0f, w, v },
		{ vtex[2].x, vtex[2].y, z, 1.0f, pInfo->LowRightColor, 0.0f, w, h },
		{ vtex[3].x, vtex[3].y, z, 1.0f, pInfo->LowLeftColor,  0.0f, u,	h }
	};
*/
	// X]
	if ( pInfo->reverse.x ) {
		vertex[0].tu = vertex[3].tu = u + w;
		vertex[1].tu = vertex[2].tu = u;
	}
	// Y]
	if ( pInfo->reverse.y ) {
		vertex[0].tv = vertex[1].tv = v + h;
		vertex[2].tv = vertex[3].tv = v;
	}

	// At@uhON
	d3d.m_pD3DDev->SetRenderState( D3DRS_ALPHABLENDENABLE, true );
	d3d.m_pD3DDev->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_MODULATE );
	d3d.m_pD3DDev->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
	d3d.m_pD3DDev->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );

	// J[uh
	switch ( pInfo->blend ) {
		// ʏuh
	case BLEND_NORMAL :
		d3d.m_pD3DDev->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
		d3d.m_pD3DDev->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );
		break;
		// Zuh
	case BLEND_ADD :
		d3d.m_pD3DDev->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
		d3d.m_pD3DDev->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_ONE );
		break;
	}

	d3d.m_pD3DDev->SetTexture( 0, d3d.texture.GetpTexture( pInfo->TexObj ) );
	d3d.m_pD3DDev->SetFVF( D3DFVF_2DVERTEX );	
	d3d.m_pD3DDev->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, 2, vertex, 
		sizeof( D3DTLVERTEX ) );
	// eNX`OFF
	d3d.m_pD3DDev->SetTexture( 0, false );	
	// At@uhOFF
	d3d.m_pD3DDev->SetRenderState( D3DRS_ALPHABLENDENABLE, false );	

	return true;
}

//-----------------------------------------------------------------------------
// Name : Delete()
// Desc : XvCg̍폜
// Argu : obj	= SPROBJ IuWFNg
//-----------------------------------------------------------------------------
bool CSpriteManager::Delete( SPROBJ &obj )
{
	if ( m_vInfo.Delete( obj ) ) return true;	
	return false;
}

//-----------------------------------------------------------------------------
// Name : GetpSpriteInfo()
// Desc : XvCg̎擾
//-----------------------------------------------------------------------------
SPRITE_INFO* CSpriteManager::GetpSpriteInfo( SPROBJ obj )
{
	return m_vInfo.GetpObject( obj );
}

//-----------------------------------------------------------------------------
// Name : SetTextureRect()
// Desc : eNX`ʒuATCY̐ݒ
//-----------------------------------------------------------------------------
bool CSpriteManager::SetTextureRect( SPROBJ obj, int x, int y, int sizex, 
									int sizey )
{
	static SPRITE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	SetRect( &pInfo->TextureRect, x, y, sizex, sizey );

	return true;
}

//-----------------------------------------------------------------------------
// Name : GetTextureRect()
// Desc : eNX`ʒuATCY̎擾
//-----------------------------------------------------------------------------
bool CSpriteManager::GetTextureRect( SPROBJ obj, RECT &rect )
{
	static SPRITE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	rect = pInfo->TextureRect;

	return true;
}

//-----------------------------------------------------------------------------
// Name : SetRotate()
// Desc : ]px̐ݒ
//-----------------------------------------------------------------------------
bool CSpriteManager::SetRotate( SPROBJ obj, float angle, float cx, float cy )
{
	static SPRITE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;
	
	pInfo->angle = angle;
	pInfo->RotateCenter.x = cx;
	pInfo->RotateCenter.y = cy;

	return true;
}

//-----------------------------------------------------------------------------
// Name : SetScale()
// Desc : XP[̐ݒ
//-----------------------------------------------------------------------------
bool CSpriteManager::SetScale( SPROBJ obj, float x, float y, float cx, float cy )
{
	static SPRITE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;
	
	pInfo->scale.x			= x;
	pInfo->scale.y			= y;
	pInfo->ScaleCenter.x	= cx;
	pInfo->ScaleCenter.y	= cy;

	return true;
}


//-----------------------------------------------------------------------------
// Name : SetBlendMode()
// Desc : uh[h̐ݒ
//-----------------------------------------------------------------------------
bool CSpriteManager::SetBlendColor( SPROBJ obj, BLEND_MODE mode, D3DCOLOR color )
{
	static SPRITE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;
	
	pInfo->blend			= mode;
	pInfo->UpLeftColor		= color;
	pInfo->UpRightColor		= color;
	pInfo->LowRightColor	= color;
	pInfo->LowLeftColor		= color;

	return true;
}

//-----------------------------------------------------------------------------
// Name : SetBlendMode()
// Desc : uh[h̐ݒ
//-----------------------------------------------------------------------------
bool CSpriteManager::SetBlendColor( SPROBJ obj, BLEND_MODE mode, 
								   D3DCOLOR UpLeft, D3DCOLOR UpRight, 
								   D3DCOLOR LowRight, D3DCOLOR LowLeft )
{
	static SPRITE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	pInfo->blend			= mode;
	pInfo->UpLeftColor		= UpLeft;
	pInfo->UpRightColor		= UpRight;
	pInfo->LowRightColor	= LowRight;
	pInfo->LowLeftColor		= LowLeft;

	return true;
}

//-----------------------------------------------------------------------------
// Name : SetReverseTexture()
// Desc : eNX`̔]
//-----------------------------------------------------------------------------
bool CSpriteManager::SetReverseTexture( SPROBJ obj, bool x, bool y )
{
	static SPRITE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	pInfo->reverse.x = x;
	pInfo->reverse.y = y;

	return true;
}