#include "../Include/AppSystem.h"

//-----------------------------------------------------------------------------
// Name : XFILE_DATA()
//-----------------------------------------------------------------------------
XFILE_DATA::XFILE_DATA()
{
	nRef = 1;

	pD3DMesh		= 0;		// wt@C[hpbV
	pTextures		= 0;		// eNX`obt@
	pD3DMaterial	= 0;		// }eAobt@
	dwMaterialNum	= 0;		// }eA
	pD3DBuffer		= 0;		// }eA
}

//-----------------------------------------------------------------------------
// Name : XFILE_INFO()
//-----------------------------------------------------------------------------
XFILE_INFO::XFILE_INFO()
{
	Info.Rotate		= D3DXVECTOR3( 0, 0, 0 );			// ]
	Info.Scale		= D3DXVECTOR3( 1.0f, 1.0f, 1.0f );	// {
	Info.Translate	= D3DXVECTOR3( 0, 0, 0 );			// sړ
	pData = 0;
	pData = new XFILE_DATA();
}

//-----------------------------------------------------------------------------
// Name : ~XFILE_INFO()
//-----------------------------------------------------------------------------
XFILE_INFO::~XFILE_INFO()
{
	RELEASE( pData );
}

//-----------------------------------------------------------------------------
// Name : AddRef()
// Desc : QƃJE^₷
// Rtn  : QƃJE^
//-----------------------------------------------------------------------------
int XFILE_DATA::AddRef()
{
	return ++nRef;
}

//-----------------------------------------------------------------------------
// Name : Release()
// Desc : ̊J
//-----------------------------------------------------------------------------
void XFILE_DATA::Release()
{
	if ( 0 != --nRef ) return;
	
	if ( !pD3DMesh ) return;
	
	// eNX`J
	if ( pTextures ) {
		for ( DWORD i = 0; i < dwMaterialNum; i++ )
			RELEASE( pTextures[i] );
		SAFE_DELETE_ARRAY( pTextures );
	}

	// }eAJ
	if ( pD3DMaterial ) {
		SAFE_DELETE_ARRAY( pD3DMaterial );
	}

	// }eÅJ
	RELEASE( pD3DBuffer );

	// bVJ
	RELEASE( pD3DMesh );

	delete this;
}

//-----------------------------------------------------------------------------
// Name : Create()
// Desc : XFile ̍쐬
// Argu : obj	= XFOBJ IuWFNg
//		  char	= t@C
//-----------------------------------------------------------------------------
bool CXFileManager::Create( XFOBJ &obj, const char *filename )
{
	LPD3DXBUFFER	pD3DBuffer	= 0;	// }eAobt@ւ̃|C^

	m_vInfo.Create( obj );
	static XFILE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;
	
	static XFILE_DATA *pData;
	pData = pInfo->pData;

	static HRESULT hr;

	// Xt@C̓ǂݍ
	hr = D3DXLoadMeshFromX( filename, D3DXMESH_MANAGED, d3d.m_pD3DDev, 
							0, &( pData->pD3DBuffer ), 0,
							&( pData->dwMaterialNum ), &( pData->pD3DMesh ) );
	if ( FAILED( hr ) ) {
		ErrLog( "CXFileManager::Create() : D3DXLoadMeshFromX()\n"
			    "		%s\n"
			    "		%s", filename, DXGetErrorString9( hr ) );
		MesBox( "G[", "%s ̓ǂݍ݂Ɏs܂", filename );
		m_vInfo.Delete( obj );
		return false;
	}
	pInfo->strFileName = filename;

	// EeNX`[̓ǂݍ
	pData->pD3DMaterial = new D3DMATERIAL9[pData->dwMaterialNum];
	pData->pTextures = new LPDIRECT3DTEXTURE9[pData->dwMaterialNum];

	D3DXMATERIAL* d3dxMaterials = ( D3DXMATERIAL* ) pData->pD3DBuffer->GetBufferPointer();

	for ( DWORD i = 0; i < pData->dwMaterialNum; i++ ) {
		// 
		pData->pD3DMaterial[i] = d3dxMaterials[i].MatD3D;
		pData->pD3DMaterial[i].Ambient = pData->pD3DMaterial[i].Diffuse;

		// eNX`Ȃꍇ
		if ( d3dxMaterials[i].pTextureFilename == 0 )
			pData->pTextures[i] = 0;
		// eNX`ꍇ
		else {
			// t@CEpX̎擾
			static char ch[256];
			strcpy( ch, filename );
			RemoveFileName( ch );
			sprintf( ch, "%s/%s", strlen( ch ) ? ch : ".", 
				d3dxMaterials[i].pTextureFilename );

			// eNX`[
			hr = D3DXCreateTextureFromFile( d3d.m_pD3DDev,
					ch, &( pInfo->pData->pTextures[i] ) );
			if ( FAILED( hr ) ) {
				ErrLog( "CXFileManager::Create() : D3DXCreateTextureFromFile()\n"
					    "		%s\n"
						"		%s", ch, DXGetErrorString9( hr ) );
				pData->pTextures[i] = 0;
			}
		}
	}

	return true;
}

//-----------------------------------------------------------------------------
// Name : Duplicate()
// Desc : XFile ̕
// Argu : dest		= 
//		  source	= 
//-----------------------------------------------------------------------------
bool CXFileManager::Duplicate( XFOBJ &dest, XFOBJ source )
{
	// A悪ꍇ͉ȂŏI
	if ( dest.n == source.n ) return true;

	static XFILE_INFO *pInfo2;
	pInfo2 = m_vInfo.GetpObject( source );
	if ( pInfo2 == 0 ) return false;

	m_vInfo.Create( dest );
	static XFILE_INFO *pInfo1;
	pInfo1 = m_vInfo.GetpObject( dest );
	if ( pInfo1 == 0 ) return false;

	// XFileInfo ̃Rs[
	*pInfo1 = *pInfo2;
	pInfo1->pData->AddRef();

	return true;
}

//-----------------------------------------------------------------------------
// Name : Delete()
// Desc : XFile ̍폜
// Argu : obj = XFOBJ IuWFNg
//-----------------------------------------------------------------------------
bool CXFileManager::Delete( XFOBJ &obj )
{
	if ( m_vInfo.Delete( obj ) ) return true;
	return false;
}

//-----------------------------------------------------------------------------
// Name : Draw()
// Desc : XFile ̕`
// Argu : obj = XFOBJ IuWFNg
//-----------------------------------------------------------------------------
bool CXFileManager::Draw( XFOBJ obj )
{
	static XFILE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	// CgON
	d3d.GetpD3DDev()->SetRenderState( D3DRS_LIGHTING, true );
	// Zobt@L
	d3d.SetRenderState( D3DRS_ZWRITEENABLE, TRUE );
	// @̐K
	d3d.GetpD3DDev()->SetRenderState( D3DRS_NORMALIZENORMALS, TRUE ); 
	
	D3DXMATRIX mWorld, mRot, mScale, mTrans;

	// ]s
	D3DXMatrixRotationYawPitchRoll( &mRot, -D3DXToRadian( pInfo->Info.Rotate.y ),
		D3DXToRadian( pInfo->Info.Rotate.x ), D3DXToRadian( pInfo->Info.Rotate.z ) );
	// XP[Os
	D3DXMatrixScaling( &mScale, pInfo->Info.Scale.x,
		pInfo->Info.Scale.y, pInfo->Info.Scale.z );
	// sړs
	D3DXMatrixTranslation( &mTrans, pInfo->Info.Translate.x, 
		pInfo->Info.Translate.y, pInfo->Info.Translate.z );
	
	mWorld = mScale * mRot * mTrans;
	d3d.m_pD3DDev->SetTransform( D3DTS_WORLD, &mWorld );

    // wt@C̕\
    for( DWORD i = 0; i < pInfo->pData->dwMaterialNum; i++ ) {
        d3d.m_pD3DDev->SetMaterial( &( pInfo->pData->pD3DMaterial[i] ) );
        d3d.m_pD3DDev->SetTexture( 0, pInfo->pData->pTextures[i] );

		static HRESULT hr;
		hr = pInfo->pData->pD3DMesh->DrawSubset( i );
		if ( FAILED( hr ) ) {
			ErrLog( "CXFileManager::Draw() : DrawSubset()\n"
					"		%s", DXGetErrorString9( hr ) );
		}
    }

	return true;
}

//-----------------------------------------------------------------------------
// Name : SetRotate()
// Desc : XFile ̉]̐ݒ
// Argu : obj = XFOBJ IuWFNg
//	      x = ẘpx
//		  y = x̊px
//		  z = ẙpx
//-----------------------------------------------------------------------------
bool CXFileManager::SetRotate( XFOBJ obj, float x, float y, float z )
{
	static XFILE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	pInfo->Info.Rotate = D3DXVECTOR3( x, y, z );

	return true;
}

//-----------------------------------------------------------------------------
// Name : SetTranslate()
// Desc : XFile ̕sړ̐ݒ
// Argu : obj = XFOBJ IuWFNg
//		  x = w̍W
//		  y = x̍W
//		  z = y̍W
//-----------------------------------------------------------------------------
bool CXFileManager::SetTranslate( XFOBJ obj, float x, float y, float z )
{
	static XFILE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	pInfo->Info.Translate = D3DXVECTOR3( x, y, z );

	return true;
}

//-----------------------------------------------------------------------------
// Name : SetScale()
// Desc : XFile ̔{̐ݒ
// Argu : obj = XFOBJ IuWFNg
//		  x = w̔{
//		  y = x̔{
//		  z = y̔{
//-----------------------------------------------------------------------------
bool CXFileManager::SetScale( XFOBJ obj, float x, float y, float z )
{
	static XFILE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	pInfo->Info.Scale = D3DXVECTOR3( x, y, z );

	return true;
}

//-----------------------------------------------------------------------------
// Name : GetRotate()
// Desc : XFile ̉]̎擾
// Argu : obj = XFOBJ IuWFNg
//	      x = ẘpx
//		  y = x̊px
//		  z = ẙpx
//-----------------------------------------------------------------------------
bool CXFileManager::GetRotate( XFOBJ obj, D3DXVECTOR3 &vec )
{
	static XFILE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	vec = pInfo->Info.Rotate;
		
	return true;
}

//-----------------------------------------------------------------------------
// Name : GetTranslate()
// Desc : XFile ̕sړ̎擾
// Argu : obj = XFOBJ IuWFNg
//		  x = w̍W
//		  y = x̍W
//		  z = y̍W
//-----------------------------------------------------------------------------
bool CXFileManager::GetTranslate( XFOBJ obj, D3DXVECTOR3 &vec )
{
	static XFILE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	vec = pInfo->Info.Translate;

	return true;
}

//-----------------------------------------------------------------------------
// Name : GetScale()
// Desc : XFile ̔{̎擾
// Argu : obj = XFOBJ IuWFNg
//		  vec = XYZ̔{
//-----------------------------------------------------------------------------
bool CXFileManager::GetScale( XFOBJ obj, D3DXVECTOR3 &vec )
{
	static XFILE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	vec = pInfo->Info.Scale;

	return true;
}

//-----------------------------------------------------------------------------
// Name : SetDeffuseColor()
//-----------------------------------------------------------------------------
bool CXFileManager::SetDeffuseColor( XFOBJ obj, D3DCOLOR color )
{
	static XFILE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	static XFILE_DATA *pData;
	pData = pInfo->pData;

	for ( DWORD i = 0; i < pData->dwMaterialNum; i++ ) {
		pData->pD3DMaterial[i].Diffuse = D3DXCOLOR( color );
	}

	return true;
}

//-----------------------------------------------------------------------------
// Name : SetAmbientColor()
//-----------------------------------------------------------------------------
bool CXFileManager::SetAmbientColor( XFOBJ obj, D3DCOLOR color )
{
	static XFILE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	static XFILE_DATA *pData;
	pData = pInfo->pData;

	for ( DWORD i = 0; i < pData->dwMaterialNum; i++ ) {
		pData->pD3DMaterial[i].Ambient = D3DXCOLOR( color );
	}

	return true;
}

//-----------------------------------------------------------------------------
// Name : SetTexture()
//-----------------------------------------------------------------------------
bool CXFileManager::SetTexture( XFOBJ obj, char *filename )
{
	static XFILE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return false;

	for ( DWORD i = 0; i < pInfo->pData->dwMaterialNum; i++ ) {
		static HRESULT hr;
		hr = D3DXCreateTextureFromFile( d3d.m_pD3DDev, filename, 
										&( pInfo->pData->pTextures[i] ) );
		if ( FAILED( hr ) ) {
			ErrLog( "CXFileManager::SetTexture() : D3DXCreateTextureFromFile()\n"
					"		%s\n"
					"		%s", filename, DXGetErrorString9( hr ) );
			pInfo->pData->pTextures[i] = 0;
		}
	}
	return true;
}

//-----------------------------------------------------------------------------
// Name : GetpXFileInfo()
// Desc : XFILE_INFO |C^̎擾
// Argu : obj = XFOBJ IuWFNg
//-----------------------------------------------------------------------------
XFILE_INFO* CXFileManager::GetpXFileInfo( XFOBJ obj )
{
	return m_vInfo.GetpObject( obj );
}

//-----------------------------------------------------------------------------
// Name : RayHit()
// Desc : CƂ̓蔻
// Argu : obj = XFOBJ IuWFNg
//-----------------------------------------------------------------------------
bool CXFileManager::RayHit( XFOBJ obj, D3DXVECTOR3 RayPos, D3DXVECTOR3 RayDir, 
		     bool *pHit, DWORD *pFaceIndex, float *pU, float *pV, float *pDist )
{
	static XFILE_INFO *pXFInfo;
	pXFInfo = m_vInfo.GetpObject( obj );
	if ( pXFInfo == 0 ) return false;

	static XFILE_DATA *pData;
	pData = pXFInfo->pData;
	D3DXMATRIX_INFO *pInfo = &(pXFInfo->Info);
	D3DXMATRIX revmat, mat, mTrans, mScale, mRot;
	D3DXMatrixIdentity(&mat);

	// sړ
	D3DXMatrixTranslation( &mTrans,
		pInfo->Translate.x, 
		pInfo->Translate.y, 
		pInfo->Translate.z );
	// gk
	D3DXMatrixScaling( &mScale, 
		pInfo->Scale.x, 
		pInfo->Scale.y, 
		pInfo->Scale.z );
	// ]
	D3DXMatrixRotationYawPitchRoll( &mRot, 
		D3DXToRadian( pInfo->Rotate.y ), 
		D3DXToRadian( pInfo->Rotate.x ), 
		D3DXToRadian( pInfo->Rotate.z ) );

	mat = mScale * mRot * mTrans;
	D3DXMatrixInverse( &revmat, 0, &mat );

	D3DXVec3TransformCoord( &RayPos, &RayPos, &revmat );
	// sړ𖳎gXtH[
	D3DXVec3TransformNormal( &RayDir, &RayDir, &revmat ); 
	BOOL bHit;

	HRESULT hr = D3DXIntersect( pData->pD3DMesh, &RayPos, &RayDir, &bHit, 
							 pFaceIndex, pU, pV, pDist, 0 , 0 );
	if ( FAILED( hr ) ) return false;

	*pHit = bHit ? true : false;

	return true;
}

//-----------------------------------------------------------------------------
// Name : GetpFileName()
// Desc : t@C̎擾
// Argu : obj = XFOBJ IuWFNg
//-----------------------------------------------------------------------------
const char* CXFileManager::GetpFileName( XFOBJ obj )
{
	static XFILE_INFO *pInfo;
	pInfo = m_vInfo.GetpObject( obj );
	if ( pInfo == 0 ) return 0;
	
	return pInfo->strFileName.c_str();
}